@@ -1,7 +1,7 @@
 /*
  * libipv6calc: Function libary for conversions
  *
- * Version:		$Id: libipv6calc.c,v 0.20 2001/04/08 $
+ * Version:		$Id: libipv6calc.c,v 1.2 2001/10/09 07:03:10 peter Exp $
  * 
  * Author:		Peter Bieringer <pb@bieringer.de>
  *
@@ -10,8 +10,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+/*
 #if defined(__NetBSD__)
+*/
 /* work around incompatibilities between Linux and KAME */
+/*
 #define s6_addr32              __u6_addr.__u6_addr32
 #define in6_u                  __u6_addr
 #define u6_addr32              __u6_addr32
@@ -21,36 +24,39 @@
 #include <linux/ipv6.h>
 #endif
 #endif 
-#include "include/modifiedfromlinuxkernel/include/net/ipv6-split.h"
-#include "include/modifiedfromlinuxkernel/net/ipv6/addrconf-split.c"
+*/
+
 #include "ipv6calc.h"
+#include "libipv6addr.h"
+#include "librfc1884.h"
+
 
 /* function stores an IPv6 address string into a structure
  *
  * in : *addrstring = IPv6 address
  * out: *resultstring = error message
- * out: ipv6addr = IPv6 address structure
+ * out: ipv6addrp = changed IPv6 address structure
  * ret: ==0: ok, !=0: error
  */
+#define DEBUG_function_name "libipv6calc/addr_to_ipv6addrstruct"
 int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6addr *ipv6addrp) {
 	int retval = 1, result, i, cpoints = 0, ccolons = 0;
 	char *addronlystring, *cp, tempstring[NI_MAXHOST];
-	struct in6_addr inet6addr;
 	int scope = 0, expecteditems = 0;
 	unsigned int temp[8];
 	int compat[4];
 
 	sprintf(resultstring, "%s", ""); /* clear result string */
 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/addr_to_ipv6addrstruct: got input %s\n", addrstring);
-#endif
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: got input %s\n", DEBUG_function_name,  addrstring);
+	};
 	
-    /* save prefix length first, if available */
+	/* save prefix length first, if available */
 	ipv6addrp->flag_prefixuse = 0; /* reset flag first */
-    addronlystring = strtok (addrstring, "/");
+	addronlystring = strtok (addrstring, "/");
 	cp = strtok (NULL, "/");
-    if ( cp != NULL ) {
+	if ( cp != NULL ) {
 		i = atol(cp);
 		if (i < 0 || i > 128 ) {
 			sprintf(resultstring, "Illegal prefix length: '%s'", cp);
@@ -59,9 +65,10 @@ int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6ad
 		};
 		ipv6addrp->flag_prefixuse = 1;
 		ipv6addrp->prefixlength = i;
-#ifdef DEBUG_libipv6calc
-		fprintf(stderr, "libipv6calc/addr_to_ipv6addrstruct: prefix length %d\n", ipv6addrp->prefixlength);
-#endif
+		
+		if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+			fprintf(stderr, "%s: prefix length %d\n", DEBUG_function_name, ipv6addrp->prefixlength);
+		};
 	};
 
 	/* uncompress string, if necessary */
@@ -99,12 +106,9 @@ int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6ad
 	for ( i = 0; i <= 7; i++ ) {
 		compat[i] = 0;
 	};
-	for ( i = 0; i <= 7 ; i++ ) {
-		ipv6addrp->addr6[i] = 0;
-	};
 
-
-	/* scan address into array */
+	ipv6addr_clear(ipv6addrp);
+	
 	if ( ccolons == 6 ) {
 		/* compatv4/mapped format */
 		expecteditems = 10;
@@ -126,9 +130,9 @@ int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6ad
 		result = sscanf(tempstring, "%x:%x:%x:%x:%x:%x:%x:%x", &temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5], &temp[6], &temp[7]);
 	};
 	
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/addr_to_ipv6addrstruct: reading into array, got items: %d\n", result);
-#endif
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: reading into array, got items: %d\n", DEBUG_function_name, result);
+	};
 
 	if ( result != expecteditems ) {
 		sprintf(resultstring, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
@@ -147,22 +151,20 @@ int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6ad
 	
 	/* copy into structure */
 	for ( i = 0; i <= 7; i++ ) {
-		ipv6addrp->addr6[i] = temp[i];
+		ipv6addr_setword(ipv6addrp, i, temp[i]);
 	};
 
-	/* get type of address */
-	for ( i = 0; i < 4; i++ ) {
-		inet6addr.s6_addr32[i] = (ipv6addrp->addr6[i <<1] <<16) | ipv6addrp->addr6[(i <<1 )+1];
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: In structure %08x %08x %08x %08x\n", DEBUG_function_name, ipv6addr_getdword(ipv6addrp, 0), ipv6addr_getdword(ipv6addrp, 1), ipv6addr_getdword(ipv6addrp, 2), ipv6addr_getdword(ipv6addrp, 3));
+		fprintf(stderr, "%s: In structure %04x %04x %04x %04x %04x %04x %04x %04x\n", DEBUG_function_name, ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
 	};
 	
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/addr_to_ipv6addrstruct: In structure %08x %08x %08x %08x\n", inet6addr.s6_addr32[0], inet6addr.s6_addr32[1], inet6addr.s6_addr32[2], inet6addr.s6_addr32[3]);
-#endif
-	
-	result = ipv6_addr_type(&inet6addr); 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/addr_to_ipv6addrstruct: Got scope %02x\n", result);
-#endif
+	result = ipv6addr_gettype(ipv6addrp); 
+
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: Got scope %02x\n", DEBUG_function_name, result);
+	};
+
 	ipv6addrp->scope = result;
 
 	if ( scope != 0 ) {
@@ -174,13 +176,14 @@ int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6ad
 		};
 	};
 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/addr_to_ipv6addrstruct: First word is: %x, address info value: %x\n", inet6addr.s6_addr32[0], result);
-#endif
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: First word is: %x, address info value: %x\n", DEBUG_function_name, ipv6addr_getword(ipv6addrp, 0), result);
+	};
 	
 	retval = 0;
 	return (retval);
 };
+#undef DEBUG_function_name
 
 
 /* function stores the ipv6addr structure in an uncompressed IPv6 format string
@@ -189,29 +192,32 @@ int addr_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6ad
  * out: *resultstring = IPv6 address (modified)
  * ret: ==0: ok, !=0: error
  */
+#define DEBUG_function_name "libipv6calc/ipv6addrstruct_to_uncompaddr"
 int ipv6addrstruct_to_uncompaddr(ipv6calc_ipv6addr *ipv6addrp, char *resultstring) {
 	int retval = 1, result;
+	char tempstring[NI_MAXHOST];
 
 	/* print array */
-	if (ipv6addrp->flag_prefixuse == 1) {
-		if ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) {
-			result = sprintf(resultstring, "%x:%x:%x:%x:%x:%x:%u.%u.%u.%u/%u", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6] >> 8, ipv6addrp->addr6[6] & 0xff, ipv6addrp->addr6[7] >> 8, ipv6addrp->addr6[7] & 0xff, ipv6addrp->prefixlength);
-		} else {
-			result = sprintf(resultstring, "%x:%x:%x:%x:%x:%x:%x:%x/%u", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6], ipv6addrp->addr6[7], ipv6addrp->prefixlength);
-		};
+	if ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) {
+		result = sprintf(tempstring, "%x:%x:%x:%x:%x:%x:%u.%u.%u.%u", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 	} else {
-		if ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) {
-			result = sprintf(resultstring, "%x:%x:%x:%x:%x:%x:%u.%u.%u.%u", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6] >> 8, ipv6addrp->addr6[6] & 0xff, ipv6addrp->addr6[7] >> 8, ipv6addrp->addr6[7] & 0xff);
-		} else {
-			result = sprintf(resultstring, "%x:%x:%x:%x:%x:%x:%x:%x", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6], ipv6addrp->addr6[7]);
-		};
+		result = sprintf(tempstring, "%x:%x:%x:%x:%x:%x:%x:%x", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
 	};
 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/ipv6addrstruct_to_uncompaddr: result string: %s\n", resultstring);
-#endif
+	if (ipv6addrp->flag_prefixuse == 1) {
+		/* append prefix length */
+		result = sprintf(resultstring, "%s/%u", tempstring, ipv6addrp->prefixlength);
+	} else {
+		result = sprintf(resultstring, "%s", tempstring);
+	};
+
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: result string: %s\n", DEBUG_function_name, resultstring);
+	};
+		
 	return (retval);
 };
+#undef DEBUG_function_name
 
 
 /* function stores the prefix of an ipv6addr structure in an uncompressed IPv6 format string
@@ -220,6 +226,7 @@ int ipv6addrstruct_to_uncompaddr(ipv6calc_ipv6addr *ipv6addrp, char *resultstrin
  * out: *resultstring = IPv6 address (modified)
  * ret: ==0: ok, !=0: error
  */
+#define DEBUG_function_name "libipv6calc/ipv6addrstruct_to_uncompaddrsuffix"
 int ipv6addrstruct_to_uncompaddrprefix(ipv6calc_ipv6addr *ipv6addrp, char *resultstring) {
 	int retval = 1;
 	int max, i;
@@ -236,27 +243,29 @@ int ipv6addrstruct_to_uncompaddrprefix(ipv6calc_ipv6addr *ipv6addrp, char *resul
 		retval = 1;
 		return (retval);
 	};
-   
+
 	max = ( ipv6addrp->prefixlength - 1 ) >> 4;
 	i = 0;
 	sprintf(tempstring1, "%s", "");
 	while (i <= max ) {
 		if ( i < max ) {
-			sprintf(tempstring2, "%s%x:", tempstring1, ipv6addrp->addr6[i]);
+			sprintf(tempstring2, "%s%x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
 		} else {
-			sprintf(tempstring2, "%s%x", tempstring1, ipv6addrp->addr6[i]);
+			sprintf(tempstring2, "%s%x", tempstring1, ipv6addr_getword(ipv6addrp, i));
 		};
 		i++;
 		sprintf(tempstring1, "%s", tempstring2);
 	};
 	sprintf(resultstring, "%s", tempstring1);
 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/ipv6addrstruct_to_uncompaddrprefix: result string: %s\n", resultstring);
-#endif
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: result string: %s\n", DEBUG_function_name, resultstring);
+	};
+
 	retval = 0;
 	return (retval);
 };
+#undef DEBUG_function_name
 
 
 /* function stores the suffix of an ipv6addr structure in an uncompressed IPv6 format string
@@ -265,6 +274,7 @@ int ipv6addrstruct_to_uncompaddrprefix(ipv6calc_ipv6addr *ipv6addrp, char *resul
  * out: *resultstring = IPv6 address (modified)
  * ret: ==0: ok, !=0: error
  */
+#define DEBUG_function_name "libipv6calc/ipv6addrstruct_to_uncompaddrsuffix"
 int ipv6addrstruct_to_uncompaddrsuffix(ipv6calc_ipv6addr *ipv6addrp, char *resultstring) {
 	int retval = 1;
 	int max, i;
@@ -281,30 +291,32 @@ int ipv6addrstruct_to_uncompaddrsuffix(ipv6calc_ipv6addr *ipv6addrp, char *resul
 		retval = 1;
 		return (retval);
 	};
-   
+
 	max = 7;
 	i   = ipv6addrp->prefixlength >> 4;
 	sprintf(tempstring1, "%s", "");
 	while (i <= max ) {
 		if ( ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) && ( i == 6 ) ) {
-			sprintf(tempstring2, "%s%u.%u.%u.%u", tempstring1, ipv6addrp->addr6[6] >> 8, ipv6addrp->addr6[6] & 0xff, ipv6addrp->addr6[7] >> 8, ipv6addrp->addr6[7] & 0xff);
+			sprintf(tempstring2, "%s%u.%u.%u.%u", tempstring1, ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 			i = max;
 		} else if ( i < max ) {
-			sprintf(tempstring2, "%s%x:", tempstring1, ipv6addrp->addr6[i]);
+			sprintf(tempstring2, "%s%x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
 		} else {
-			sprintf(tempstring2, "%s%x", tempstring1, ipv6addrp->addr6[i]);
+			sprintf(tempstring2, "%s%x", tempstring1, ipv6addr_getword(ipv6addrp, i));
 		};
 		i++;
 		sprintf(tempstring1, "%s", tempstring2);
 	};
 	sprintf(resultstring, "%s", tempstring1);
 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/ipv6addrstruct_to_uncompaddrsuffix: result string: %s\n", resultstring);
-#endif
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: result string: %s\n", DEBUG_function_name, resultstring);
+	};
+	
 	retval = 0;
 	return (retval);
 };
+#undef DEBUG_function_name
 
 
 /* function stores the ipv6addr structure in an full uncompressed IPv6 format string
@@ -313,29 +325,32 @@ int ipv6addrstruct_to_uncompaddrsuffix(ipv6calc_ipv6addr *ipv6addrp, char *resul
  * out: *resultstring = IPv6 address (modified)
  * ret: ==0: ok, !=0: error
  */
+#define DEBUG_function_name "libipv6calc/ipv6addrstruct_to_fulluncompaddr"
 int ipv6addrstruct_to_fulluncompaddr(ipv6calc_ipv6addr *ipv6addrp, char *resultstring) {
 	int retval = 1, result;
+	char tempstring[NI_MAXHOST];
 
 	/* print array */
-	if (ipv6addrp->flag_prefixuse == 1) {
-		if ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) {
-			result = sprintf(resultstring, "%04x:%04x:%04x:%04x:%04x:%04x:%u.%u.%u.%u/%u", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6] >> 8, ipv6addrp->addr6[6] & 0xff, ipv6addrp->addr6[7] >> 8, ipv6addrp->addr6[7] & 0xff, ipv6addrp->prefixlength);
-		} else {
-			result = sprintf(resultstring, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x/%u", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6], ipv6addrp->addr6[7], ipv6addrp->prefixlength);
-		};
+	if ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) {
+		result = sprintf(tempstring, "%04x:%04x:%04x:%04x:%04x:%04x:%u.%u.%u.%u", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 	} else {
-		if ( ( ipv6addrp->scope & IPV6_ADDR_COMPATv4 ) || ( ipv6addrp->scope & IPV6_ADDR_MAPPED ) ) {
-			result = sprintf(resultstring, "%04x:%04x:%04x:%04x:%04x:%04x:%u.%u.%u.%u", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6] >> 8, ipv6addrp->addr6[6] & 0xff, ipv6addrp->addr6[7] >> 8, ipv6addrp->addr6[7] & 0xff);
-		} else {
-			result = sprintf(resultstring, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x", ipv6addrp->addr6[0], ipv6addrp->addr6[1], ipv6addrp->addr6[2], ipv6addrp->addr6[3], ipv6addrp->addr6[4], ipv6addrp->addr6[5], ipv6addrp->addr6[6], ipv6addrp->addr6[7]);
-		};
+		result = sprintf(tempstring, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
 	};
 
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/ipv6calc_ipv6addr2uncomp: result string: %s\n", resultstring);
-#endif
+	if (ipv6addrp->flag_prefixuse == 1) {
+		/* append prefix length */
+		result = sprintf(resultstring, "%s/%u", tempstring, ipv6addrp->prefixlength);
+	} else {
+		result = sprintf(resultstring, "%s", tempstring);
+	};
+
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: result string: %s\n", DEBUG_function_name, resultstring);
+	};
+	
 	return (retval);
-}
+};
+#undef DEBUG_function_name
 
 
 /* mask prefix bits (set suffix bits to 0)
@@ -343,12 +358,14 @@ int ipv6addrstruct_to_fulluncompaddr(ipv6calc_ipv6addr *ipv6addrp, char *results
  * in:  structure via reference
  * out: modified structure
  */
+#define DEBUG_function_name "libipv6calc/ipv6addrstruct_masksuffix"
 void ipv6addrstruct_maskprefix(ipv6calc_ipv6addr *ipv6addrp) {
 	int nbit, nword;
 	unsigned int mask, newword;
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/ipv6calc_ipv6addr_maskprefix: called\n");
-#endif
+
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: called\n", DEBUG_function_name);
+	};
    
 	if (ipv6addrp->flag_prefixuse != 1) {
 		/* hmm, no prefix specified. skip */
@@ -364,15 +381,17 @@ void ipv6addrstruct_maskprefix(ipv6calc_ipv6addr *ipv6addrp) {
 				 
 			/* calculate mask */
 			mask = 0x8000 >> ((nbit & 0x0f));
-			newword = ipv6addrp->addr6[nword] & (~ mask );
+			newword = ipv6addr_getword(ipv6addrp, nword) & (~ mask );
 			
-#ifdef DEBUG_libipv6calc
-			fprintf(stderr, "libipv6calc/ipv6calc_ipv6addr_maskprefix: bit: %d = nword: %d, mask: %04x, word: %04x newword: %04x\n", nbit, nword, mask, ipv6addrp->addr6[nword], newword);
-#endif
-			ipv6addrp->addr6[nword] = newword;
+			if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+				fprintf(stderr, "%s: bit: %d = nword: %d, mask: %04x, word: %04x newword: %04x\n", DEBUG_function_name, nbit, nword, mask, ipv6addr_getword(ipv6addrp, nword), newword);
+			};
+
+			ipv6addr_setword(ipv6addrp, nword, newword);
 		};
 	};
-}
+};
+#undef DEBUG_function_name
 
 
 /* mask suffix bits (set prefix bits to 0) 
@@ -380,12 +399,14 @@ void ipv6addrstruct_maskprefix(ipv6calc_ipv6addr *ipv6addrp) {
  * in:  structure via reference
  * out: modified structure
  */
+#define DEBUG_function_name "libipv6calc/ipv6addrstruct_masksuffix"
 void ipv6addrstruct_masksuffix(ipv6calc_ipv6addr *ipv6addrp) {
 	int nbit, nword;
 	unsigned int mask, newword;
-#ifdef DEBUG_libipv6calc
-	fprintf(stderr, "libipv6calc/ipv6calc_ipv6addr_masksuffix: called\n");
-#endif
+
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: called\n", DEBUG_function_name);
+	};
    
 	if (ipv6addrp->flag_prefixuse != 1) {
 		/* hmm, no prefix specified. skip */
@@ -401,14 +422,77 @@ void ipv6addrstruct_masksuffix(ipv6calc_ipv6addr *ipv6addrp) {
 				 
 			/* calculate mask */
 			mask = 0x8000 >> ((nbit & 0x0f));
-			newword = ipv6addrp->addr6[nword] & (~ mask );
+			newword = ipv6addr_getword(ipv6addrp, nword) & (~ mask );
 			
-#ifdef DEBUG_libipv6calc
-			fprintf(stderr, "libipv6calc/ipv6calc_ipv6addr_masksuffix: bit: %d = nword: %d, mask: %04x, word: %04x newword: %04x\n", nbit, nword, mask, ipv6addrp->addr6[nword], newword);
-#endif
-			ipv6addrp->addr6[nword] = newword;
+			if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+				fprintf(stderr, "libipv6calc/ipv6calc_ipv6addr_masksuffix: bit: %d = nword: %d, mask: %04x, word: %04x newword: %04x\n", nbit, nword, mask, ipv6addr_getword(ipv6addrp, nword), newword);
+			};
+
+			ipv6addr_setword(ipv6addrp, nword, newword);
 		};
 	};
 };
+#undef DEBUG_function_name
 
 
+/* function stores an 16 char identifier/token into a structure
+ *
+ * in : *addrstring = 16 char identifier/token
+ * out: *resultstring = error message
+ * out: ipv6addr = IPv6 address structure
+ * ret: ==0: ok, !=0: error
+ */
+#define DEBUG_function_name "libipv6calc/identifier_to_ipv6addrstruct"
+int identifier_to_ipv6addrstruct(char *addrstring, char *resultstring, ipv6calc_ipv6addr *ipv6addrp) {
+	int retval = 1, result, i;
+	unsigned int temp[8];
+
+	sprintf(resultstring, "%s", ""); /* clear result string */
+
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: got input '%s'\n", DEBUG_function_name, addrstring);
+	};
+	
+	if ( strlen(addrstring) != 16 ) {
+		sprintf(resultstring, "Error, given identifier/token '%s' is not valid (length != 16!", addrstring);
+		retval = 1;
+		return (retval);
+	};
+
+	/* clear variables */
+	ipv6addr_clear(ipv6addrp);
+
+	/* scan address into array */
+	result = sscanf(addrstring, "%04x%04x%04x%04x", &temp[4], &temp[5], &temp[6], &temp[7]);
+	if ( result != 4 ) {
+		sprintf(resultstring, "Error splitting address '%s', got %d items instead of 4!", addrstring, result);
+		retval = 1;
+		return (retval);
+	};
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: reading into array, got items: %d\n", DEBUG_function_name, result);
+	};
+
+	/* check address words range */
+	for ( i = 4; i <= 7; i++ ) {
+		if ( ( temp[i] < 0x0 ) || ( temp[i] > 0xffff ) )	{
+			sprintf(resultstring, "Error, given address '%s' is not valid on position %d (%x)!", addrstring, i, temp[i]);
+			retval = 1;
+			return (retval);
+		};
+	};
+	
+	/* copy into structure */
+	ipv6addr_setword(ipv6addrp, 4, temp[4]);
+	ipv6addr_setword(ipv6addrp, 5, temp[5]);
+	ipv6addr_setword(ipv6addrp, 6, temp[6]);
+	ipv6addr_setword(ipv6addrp, 7, temp[7]);
+	
+	if ( ipv6calc_debug & DEBUG_libipv6calc ) {
+		fprintf(stderr, "%s: Got value '%04x%04x%04x%04x'\n", DEBUG_function_name, temp[4], temp[5], temp[6], temp[7]);
+	};
+
+	retval = 0;
+	return (retval);
+};
+#undef DEBUG_function_name
